@@ -6,12 +6,11 @@
 # Some code based on nhdplusextractor by Mitchell Sawtelle
 #   (https://github.com/Msawtelle/PyNHDPlus)
 #
-# last updated: 08/10/2022
+# last updated: 09/25/2022
 #
 # contains the MesonetDownloader class, which can be used to retrieve and
 # manipulate source data from the Mesonet dataset online
-import urllib.request
-import os
+import urllib.request, os, pytz
 import pandas as pd
 import numpy as np
 from datetime import timedelta, datetime
@@ -36,14 +35,14 @@ class MesonetDownloader:
             verbose (bool): if true write detailed debugging to stdout
         '''
         self.verbose = verbose
+        # this is used for handling changing the rain column to differential
+        self.errorcodes=[-994,-995,-996,-997,-998,-999]
         # if no destination given in init method set destination to current
         #   directory
         if destination is None:
             self.destination = os.path.dirname(__file__)
         else:
-            self.destination = destination
-        # replace slashes in Windows style paths
-        self.destination = self.destination.replace('\\','/')
+            self.destination = os.path.normpath(destination)
 
         try:
             if not os.path.isdir(self.destination): os.mkdir(self.destination)
@@ -218,8 +217,32 @@ class MesonetDownloader:
                     print('This means that there is no data available for the'
                           ' {} on {}.'.format(site_id,i))
             if not cur_df.empty:
-                cur_df['DATE'] =  datetime.strptime(i, '%Y%m%d')
-                cur_df['DATETIME'] = cur_df.apply(lambda x: x['DATE']+timedelta(minutes = int(x['TIME'])), axis = 1)
+                date =  datetime.strptime(i, '%Y%m%d')
+                cur_df['DATETIME'] = cur_df.apply(lambda x: pytz.utc.localize(date+timedelta(minutes = int(x['TIME']))), axis = 1)
+                # change rain to differential instead of cumulative
+                # make a copy of the rain column so we can track the errors
+                cur_df['RAIN_BAK'] = cur_df['RAIN']
+                                # replace the errors with nan
+                for code in self.errorcodes:
+                    cur_df['RAIN'].replace(str(code),np.nan,inplace=True)
+                    cur_df['RAIN'].replace(code,np.nan,inplace=True)
+                first_rain = cur_df.loc[0,'RAIN']
+                second_rain = cur_df.loc[1,'RAIN']
+                # there are cases where the last reading from the previous day
+                # rolls over so we check for that and zero out the number
+                if first_rain > second_rain:
+                    first_rain = 0
+                    cur_df.loc[0,'RAIN'] = first_rain
+                cur_df['RAIN']=cur_df['RAIN'].diff()
+                cur_df.loc[0,'RAIN'] = first_rain
+                # copy error codes back into the rain column
+                # TODO: fix the SettingWithCopyError warning
+                pd.options.mode.chained_assignment = None
+                for code in self.errorcodes:
+                    cur_df['RAIN'].loc[cur_df['RAIN_BAK']==code] = code
+                pd.options.mode.chained_assignment = 'warn'
+                # drop some columns to save memory
+                cur_df.drop(columns=['STNM','TIME','RAIN_BAK'],inplace=True)
                 df_list.append(cur_df)
             elif self.verbose:
                 print('No data available for {} on {}.'.format(site_id,i))
